/*---------------------------------------------------------------
This file is part of the Buildable Evolution Software Project. 

http://www.demo.cs.brandeis.edu/pr/buildable
(C) Pablo Funes and Jordan B. Pollack 1997-2008
pablo@cs.brandeis.edu

The methods and configurations incorporated in the Builable Evolution
Software Project are covered in part by United States patent
6,434,492 "Computer apparatus and method for analyzing structural
stability" of August 13, 2002. 

The Buildable Evolution Software Project is free software: you 
can redistribute it and/or modify it under the terms of the 
GNU General Public License as published by the Free Software 
Foundation, either version 3 of the License, or (at your option) 
any later version. 

The Buildable Evolution Software Project is distributed in the
hope that it will be useful, but WITHOUT ANY WARRANTY; without
even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE. See the GNU General Public License for more
details. 

You should have received a copy of the GNU General Public License 
along with the software. If not, see <http://www.gnu.org/licenses/>. 
---------------------------------------------------------------*/
/* #define PRINTS        */

#include <stdio.h>
#include <math.h>

#include "vector.h"
#include "pfit.h"

REAL addforce(); /* forward declared */

REAL reactionforce(joint *j, int sign, body *b, vector2d x, force2d f) {
  torque2d tau,jt;
  force2d jf;
  REAL lambda,a;
  int i,i1;

  if (b == NULL) {i=0;i1=1;} else {i=b->n;i1=b->equilibrium;}
  /*
  printf("+reactionforce joint %d (%d) to body %d eq %d (%f,%f) at (%f,%f)\n",
	j->n,sign,i,i1,x.x,x.y,f.x,f.y);
	*/

  if (b != NULL)
      if (! b->equilibrium) return 0.0 ;

  tau = sign * torqueat(j->x,x,f);               /* calc torque */
  f = mul2d((REAL)sign,f);                             /* sign of force over joint */
  /*
  printf("tau = %f\n",tau);
  */
  jt = j->torques + tau;                  /* calc new joint stresses */
  jf = sum2d(j->forces,f);
  lambda = 1.0;
  if (fabs(jt) > j->maxtorque && tau != 0.0) { /* can't take all; see how much */
    jt = j->maxtorque * SIGN(tau);
    a = (jt - j->torques) / tau ;
    tau = a * tau;
    f = mul2d(a,f);
    lambda *= a;
  }
  /* now f and tau are compatible with this joint */
  
  if (reallyzero2d(f)) return 0.0;
  a = addforce(b,x,mul2d((REAL)sign,f));
  /*
  printf("reactionforce: addforce returned %f\n",a);
  */
  if (a < 1) {
    f = mul2d(a,f);
    tau = a * tau;
    lambda *= a;
  }
  j->torques = j->torques + tau;
  j->forces = sum2d(j->forces,f);
  return lambda;
}    

/* apply a force f to body b at point x */

#ifdef USEALARM
extern int ringing;
#endif

extern int addforcecount ONEFOREACH;

REAL addforce(body *b, vector2d x, force2d f) {
  USEMINE
  force2d reaction;
  torque2d tau;
  REAL lambda,a;
  int i;

  if (b==NULL) i=0; else i=b->n;
#ifdef PRINTS
  printf("addforce %d x = (%f,%f) f = (%f %f)\n",i,x.x,x.y,f.x,f.y);
#endif
  if (b == NULL) return 1.0 ;/* ground always resists */
  if (reallyzero2d(f)) return 1.0 ;

#ifdef USEALARM
  if (ringing) return 0.0;   /* alarm sounded, abort. */
#endif

  if (addforcecount MINE == 0) return 0.0; /* count exhausted, abort */
  else addforcecount MINE--;
  

  tau = torqueat(b->x, x, f);
  lambda = 0.0;
  b->equilibrium = FALSE;
  for (i=0; i < b->jointcount; i++) {
    a = reactionforce(b->joints[i],b->jointsign[i],b->joins[i],x,f);
    /*
    printf("reactionforce %d (%f,%f) f= (%f,%f) returned %f\n",
	i,x.x,x.y,f.x,f.y);
	*/
    f = sub2d(f,mul2d(a,f));
    lambda = lambda + a - a * lambda; 
    /*
    printf("newf (%f,%f) lambda=%f\n",f.x,f.y,lambda);
    */
    if (reallyzero(1-lambda)) {
      b->equilibrium = TRUE;
      return 1.0;
    }
  }
  b->equilibrium = TRUE;
  return lambda;
}

void addjoint(body *b1, int sign, body *b2, joint *j) {
  if (b1==NULL) return;
  b1->joints[b1->jointcount]=j;
  b1->joins[b1->jointcount]=b2;
  b1->jointsign[b1->jointcount]=sign;
  b1->jointcount++;
}

#ifdef UNDEF
main(int argc, char **argv) {
  int bodycount;
  int jointcount;
  body *bodies,*body1,*body2;
  joint *joints;
  int i,i1,i2,sign;
  REAL a;

  scanf ("%f %f",&grav2d.x, &grav2d.y);
  fprintf(stderr,"Gravity = (%f,%f)\n",grav2d.x,grav2d.y);
  scanf ("%d",&bodycount);
  fprintf(stderr,"%d bodies \n",bodycount);
  bodies=(body *)calloc(sizeof *bodies,bodycount);
  for (i=0;i<bodycount;i++) {
    scanf("%*d %f %f %f",&bodies[i].x.x,
	  &bodies[i].x.y,
	  &bodies[i].mass);
    bodies[i].jointcount = 0;
    bodies[i].equilibrium = TRUE;
    bodies[i].n = i + 1;
    printf("body %d at (%f,%f) mass %f\n",
	   i+1,bodies[i].x.x,bodies[i].x.y,bodies[i].mass);
  }
  
  scanf("%d",&jointcount);
  joints = (joint *) calloc(sizeof *joints,jointcount);
  fprintf(stderr,"%d joints\n",jointcount);
  for (i=0;i<jointcount;i++) {
    scanf("%*d %f %f %f %d %d", 
	  &joints[i].x.x, &joints[i].x.y,
	  &joints[i].maxtorque,
	  &i1,&i2);
    joints[i].torques = 0;
    joints[i].forces.x = 0.0;
    joints[i].forces.y = 0.0;
    joints[i].n = i+1;
    if (i1 > 0) body1 = &bodies[i1-1];
    else body1 = NULL;
    if (i2 > 0) body2 = &bodies[i2-1];
    else body2 = NULL;
    if (body1 != NULL) 
      sign = (body1->x.y > joints[i].x.y) ? 1 : -1;
    else if (body2 != NULL) 
      sign = (body2->x.y > joints[i].x.y) ? -1 : 1;
    else
      sign = 1;
    addjoint(body1,sign,body2,&joints[i]);
    addjoint(body2,-sign,body1,&joints[i]);
    printf("joint %d at (%f,%f) maxt %f %d<->%d\n",
	   i,joints[i].x.x,joints[i].x.y,joints[i].maxtorque,
	   i1,i2);
  }
    
    i1=0;
    for (i=0;i<bodycount;i++) {
	a=addforce(&bodies[i], bodies[i].x,mul2d(bodies[i].mass,grav2d));
	fprintf(stderr,"addforce %d: %f\n",i+1,a);
	if (a<1.0) {
		if (i1==0) i1 = i +1;
		printf("FAILED AT JOINT %d\n",i+1);
		break;
		}
	}

    for (i=0; i<jointcount;i++) {
	fprintf(stderr,"joint %d torques %f forces (%f,%f) maxtorque %f\n",
	    i+1,
	    joints[i].torques,joints[i].forces.x,joints[i].forces.y,
	    joints[i].maxtorque);
	}
    if (argc>0) 
      mfile(argv[1],bodies,bodycount,joints,jointcount);
}

mfile(char *name, body bodies[], int bodycount,
      joint joints[], int jointcount) {
  int i,j;
  body *b;
  joint *joint;
  FILE *fp;
  fp=fopen(name,"w");
  fprintf(fp,"figure; hold on\n");
  for (i=0,b=bodies;i<bodycount;i++,b++) {

    fprintf(fp,"plot(%f,%f,'co');\n",
	    b->x.x,b->x.y);
    
    for (j=0;j< b->jointcount;j++) {
      fprintf(fp,"plot([%f %f],[%f %f],'%s');\n",
	      b->x.x,(b->joints[j])->x.x,
	      b->x.y,(b->joints[j])->x.y,
	      reallyzero((b->joints[j])->torques * (b->jointsign[j]) - 
			 (b->joints[j])->maxtorque) ?
	      "r-":"y-");
    }
  }

  for (i=0;i<jointcount;i++) 
    fprintf(fp,"plot(%f,%f,'w*');\n",
	    joints[i].x.x,joints[i].x.y);
}
#endif


