/*---------------------------------------------------------------
This file is part of the Buildable Evolution Software Project. 

http://www.demo.cs.brandeis.edu/pr/buildable
(C) Pablo Funes and Jordan B. Pollack 1997-2008
pablo@cs.brandeis.edu

The methods and configurations incorporated in the Builable Evolution
Software Project are covered in part by United States patent
6,434,492 "Computer apparatus and method for analyzing structural
stability" of August 13, 2002. 

The Buildable Evolution Software Project is free software: you 
can redistribute it and/or modify it under the terms of the 
GNU General Public License as published by the Free Software 
Foundation, either version 3 of the License, or (at your option) 
any later version. 

The Buildable Evolution Software Project is distributed in the
hope that it will be useful, but WITHOUT ANY WARRANTY; without
even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE. See the GNU General Public License for more
details. 

You should have received a copy of the GNU General Public License 
along with the software. If not, see <http://www.gnu.org/licenses/>. 
---------------------------------------------------------------*/
#include <math.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h> // for memcpy
#include <time.h>
#include <unistd.h>

#define BYTEINT signed char

#include "vector.h"
#include "pfit.h"

/* #define MAKELINEAGE */

/* #define GLIB */

/* #define PRINTS         */

#define RUSH
#define NOCOMPRESS 1
#define NOCROSSOVER 0

#define SRANDOM time(NULL)

#ifdef PARALLEL
#define USE_RAND48
/* random library is not parallel- compatible, 
   but rand_48 seems to work well */
#endif

#ifdef USE_RAND48
#define RANDINT lrand48
#define RANDSEED srand48
#else
#define RANDINT random
#define RANDSEED srandom
#endif

#define Kilo 1000
#define Mega (Kilo*Kilo)

#define POP_SIZE 1000

#define ALPHA 2.0

#define WATCH_COUNT 10000

#define TARGET_FITNESS 40

#define ADDFORCE_ABORTTIME 0

#define HEIGHT 1.1954 /* height of a brick, in width units */
#define WEIGHT 0.3958 /* weight of a unit mass */
#define LENGTH 8.0    /* width of a unit, in mm */


/* #define PRINTS */

#define PREDEF_BRICK_NO 5
#define MAXBRICKS 127 
#define MAXBODIES 127
#define MAXJOINTS (MAXBODIES*(BODYJOINTS/2+1))

#define MAXADDFORCECOUNT 4*100*Kilo
#define COUNTLIMIT /* 150000 */ 4*Mega

/* situation description */

#define TARGET_LOAD (250.0 / WEIGHT) /*  grams */
#define LOAD_STEPS 10

#define PLATE /* 32 */ 17

#define TARGETX /* (-150) */ (-62) 
#define TARGETY (0)
#define MINY (-1)
#define MAXY 40
#define MAXX (PLATE+5)
#define MINX (- 50)

#define EMPTY (-2)
#define GROUND (-1)
#define TRANSIT (-3)
#define UNUSED (-4)
#define RESERVED (-5)

#define ISSOLID(thing) (((thing)>=0) || ((thing) == GROUND))
#define ISNBRICK(thing) (((thing) == EMPTY) || (thing)==GROUND)
#define ISBRICK(thing) (!ISNBRICK(thing))
#define ISEMPTY(thing) ((thing)==EMPTY)

#ifdef PARALLEL

int hasforked=0;
int haslocked ONEFOREACH;

void my_lock() {
	USEMINE
	if (hasforked && (haslocked MINE == 0)) m_lock();
	haslocked MINE ++;
#ifdef CHECKS
	if (haslocked MINE > 5) printf("%d:too many locks\n",m_get_myid());
#endif
}

void my_unlock() {
	USEMINE
	haslocked MINE --;
	if (hasforked && (haslocked MINE ==0)) m_unlock();
#ifdef CHECKS
	if (haslocked MINE < 0) printf("%d:lock underflow\n",m_get_myid());
#endif
}

#endif

/* addforce abortion mechanism */
int addforcecount ONEFOREACH;
unsigned long ringcount=0;

vector2d grav2d = { 0.0, -1.2 };

typedef struct {
	BYTEINT size;
	short joints[4];
	BYTEINT jointsize[4];
} brick;

brick bricks ONEFOREACH[MAXBRICKS];

joint joints ONEFOREACH [MAXJOINTS];
body bodies ONEFOREACH [MAXBODIES];

REAL totalmass ONEFOREACH;
int brickcount ONEFOREACH;
int jointcount ONEFOREACH;
int bodycount ONEFOREACH;
REAL fitnessvalue ONEFOREACH;

/* predefined bricks */

struct {
	brick brick;
	int x,y;
} predef_bricks[PREDEF_BRICK_NO] = 
{ {6, {-2,-2, -2,-2}, {0,0,0,0}, 6,0},
		{4, {-2,-2, -2,-2}, {0,0,0,0}, 12,0},
		{4, {-2,-2, -2,-2}, {0,0,0,0}, 12,1},
		{4, {-2,-2, -2,-2}, {0,0,0,0}, 12,2},
		{4, {-2,-2, -2,-2}, {0,0,0,0}, 12,3}};


#define GRID BYTEINT
#define GMAXX (MAXX+1)
#define GMINX (MINX-1)
#define GMAXY (MAXY+1)
#define GMINY (MINY-1)

GRID grid ONEFOREACH [(GMAXX+1 - GMINX) * (GMAXY+1 - GMINY)];
BYTEINT vgrid ONEFOREACH [(GMAXX+1-GMINX)] ;

#define XSHIFT(j,js,bs,nbs) (((j)==0||(j)==3)?((js)-(nbs)):((bs)-(js)))
#define YSHIFT(j) (((j)==0||(j)==1)?-1:1)


#define GRIDOFF(x,y) (((y)-GMINY)+(GMAXY-GMINY)*((x)-GMINX))

int getgrid(int x, int y) {
	USEMINE
	if (vgrid MINE [x-GMINX]==0) return EMPTY;
	else return grid MINE [GRIDOFF(x,y)];
}

int setgrid(int x, int y, int value) {
	USEMINE
	register int i;
	register int j;

	if (vgrid MINE [x-GMINX]==0) {
		for (j=GRIDOFF(x,GMINY),i=0;i<(GMAXY-GMINY);i++) 
			grid MINE [j++]=EMPTY;
		vgrid MINE [x-GMINX]=1;
	}
	x = GRIDOFF(x,y);
	y=(GRID) grid MINE [x];
	grid MINE [x]=value;
	return y;
}

void reserve_spots(int mode) {
	int i;
	for (i=6;i<12;i++) setgrid(i,0,mode);
	for (i=0;i<4;i++) setgrid(12,i,mode);
}

void initgrid() {
	USEMINE
	register int i;
	for (i=0;i< (sizeof (vgrid MINE)); i++)
		vgrid MINE [i]=0;
	for (i=0;i<PLATE;i++)
		setgrid(i,-1,GROUND);
	reserve_spots(RESERVED);
}

void makejoint(int body1, int body2, int sign, int jointsize, REAL x, REAL y) {
	USEMINE
	joint *j;
#ifdef PRINTS
printf("makejoint %d %d %d %d %f %f\n",body1,body2,sign,jointsize,x,y);
#endif
j=&joints MINE [jointcount MINE ++];
if (jointcount MINE == MAXJOINTS) {
	fprintf(stderr, "Error: Too many joints!\n");
	exit(1);
}
j->x.x = x;
j->x.y = y;
if (jointsize < 0) {
	fprintf(stderr, "Error: negative jointsize\n");
	exit(1);
}
switch(jointsize) {

case 0: j->maxtorque = 0.0; break;
case 1: j->maxtorque = 405.0; break;
case 2: j->maxtorque = 1960.0; break;
case 3: j->maxtorque = 3500.0; break;
case 4: j->maxtorque = 6144.0 /*9600.0*/; break;
case 5: j->maxtorque = 11000.0 /* 12672.0 */; break;
case 6: j->maxtorque = 13520.0; break;
case 7: j->maxtorque = 14238.0; break;
default:j->maxtorque = jointsize * 14238.0 / 7.0;
}
/* j->maxtorque = jointsize * TORQ_UNIT */;
j->n = jointcount MINE;
addjoint(&bodies MINE [body1], sign, 
		((body2==GROUND) ? NULL : &bodies MINE[body2]),j);
if (body2 != GROUND)
	addjoint(&bodies MINE [body2], -sign, &bodies MINE [body1],j);
}

int newjoints(int gx, int gy, int bodyno, int size, int sign) {
	int k,found,x,at=0;
	found=EMPTY;
	for (k=0;k<size;k++,gx++) {
		x=getgrid(gx,gy+sign);
		if (found != EMPTY && x != found && ISSOLID(found)) {
			makejoint(bodyno,found,sign, gx - at,
					((REAL) at + gx) / 2.0,
					(gy + 0.5 + 0.5 * sign) * HEIGHT);
		}
		if (found == EMPTY) at = gx;
		found=x;
	}
	if (ISSOLID(found)) 
		makejoint(bodyno,found,sign,gx-at,
				((REAL) at+ gx) / 2.0,
				(gy + 0.5 + 0.5 * sign) * HEIGHT);
	return 0;      /* never fails */
}


int putbrickat(int gx, int gy, int size) {
	USEMINE
	int j;
	/* prohibit to have bricks 'under the table' */
	if (gy < 0 && gx + size >= 0)
		return 1;
	/* or out of bounds in general */
	if (gy >= MAXY || gy < MINY || gx < MINX || gx+size >= MAXX)
		return 1;
	/* or below the diagonal */
	if (gy < (-gx)-1)
		return 1; 
	/* see if this brick fits and add it to the grid */
	for (j=0;j<size;j++) 
		if (EMPTY != setgrid(gx+j,gy,bodycount MINE)) 
			return 1;
	return 0;
}



int addbrick(int gx, int gy, int brickno) {
	USEMINE
	int j,k;
#ifdef PRINTS
	printf("AB %d %d %d(%d) ",gx,gy,brickno,bricks MINE [brickno].size);
#endif


	/* see if this brick fits and add it to the grid */
	if (putbrickat(gx,gy,bricks MINE [brickno].size))
		return 1; /* if it don't fit, you must acquit */

	/* create body */
	bodies MINE [bodycount MINE ].x.x = 0.5 * bricks MINE[brickno].size + gx;
	bodies MINE [bodycount MINE ].x.y = HEIGHT * (gy + 0.5);
	bodies MINE [bodycount MINE ].mass = bricks MINE[brickno].size;
	bodies MINE [bodycount MINE ].jointcount = 0;
	bodies MINE [bodycount MINE] .n = brickno;
	totalmass MINE += bodies MINE [bodycount MINE].mass;

	if (bodycount MINE== MAXBODIES) return 1;    /* failed: too many bodies */
	/* create joints */
	if (newjoints(gx,gy,bodycount MINE,bricks MINE[brickno].size,-1) ||
			newjoints(gx,gy,bodycount MINE,bricks MINE[brickno].size,1))
		return 1;                   /* failed */

	bodycount MINE++;

	/* create linked bricks */
	for (j=0;j<4;j++) {
		if ((k=bricks MINE[brickno].joints[j]), ISBRICK(k)) {
#ifdef PRINTS
			printf("J %d(%d) k=%d(%d) ",j,bricks MINE[brickno].jointsize[j],k,
					bricks MINE[k].size);
#endif
			if (addbrick (gx + XSHIFT(j,bricks MINE[brickno].jointsize[j],
					bricks MINE[brickno].size,
					bricks MINE[k].size),
					gy + YSHIFT(j),k))
				return 1;    /* failed */
		}
	}
	return 0;
}  






int makemodel() {
	USEMINE
	int ret;
	int i,c;
#ifdef PRINTS
	printf("MM ");
#endif
	initgrid();


	jointcount MINE = 0;
	bodycount MINE = 0;
	totalmass MINE= 0.0;
	ret = addbrick(0,0,0);
#ifdef PRINTS
	printf("ab000=%d ",ret);
#endif
	if (ret==0) {
		/* add predefined bricks */
		c=brickcount MINE;
		reserve_spots(EMPTY); /* de-regulate previously reserved spots */
		for (i=0;i<PREDEF_BRICK_NO;i++) {
			bricks MINE [i+c] =  predef_bricks[i].brick;
			if (addbrick(predef_bricks[i].x,predef_bricks[i].y,i+c)) {
				fprintf(stderr,"Error adding predefs\n");
				/* exit(1); */ return 1;
			}
		}
		brickcount MINE = c +  PREDEF_BRICK_NO;
	}
#ifdef PRINTS
	if (ret) printf ("MM Failed\n");
	else printf("\n");
#endif

	return ret;
}

int tooheavy() {
	USEMINE
	/* init joints and bodies */
	int i;

	for (i=0; i<bodycount MINE;i++)
		bodies MINE[i].equilibrium = 1;
	for (i=0;i<jointcount MINE;i++) {
		joints MINE [i].torques = 0.0;
		joints MINE [i].forces.x = joints MINE [i].forces.y = 0.0;
	}
	addforcecount MINE =MAXADDFORCECOUNT;
	for (i=0;i<bodycount MINE;i++) {
#ifdef PRINTS
		printf("addforce body %d of %d...\n",i,bodycount MINE);
#endif
		if (.9999 > 
		addforce(&bodies MINE[i],bodies MINE[i].x,
				mul2d(bodies MINE[i].mass,grav2d))) {
			if (addforcecount MINE== 0) {
				INCREMENT(ringcount);
#ifdef PRINTS
				printf("AAAddforce aborted\n");
#endif
			}
#ifdef PRINTS
			printf("Too heavy.\n");
#endif
			return 1;
		}
	}
	return 0;
}


REAL fitness() {
	USEMINE
	int i,k,where=0;
	REAL ret;
	REAL minx,u,miny=0.0;
	int n;
	vector2d loadvector;
	vector2d cranex;

	/* find lefttmost brick */

	minx = 10.0;
	for (i=0;i<bodycount MINE;i++) {
		n = bodies MINE [i].n;
		if ((u=(bodies MINE [i].x.x - 0.5 * bricks MINE [n].size)) < minx) {
			minx = u;
			miny=bodies MINE [i].x.y;
			where=i;
		}
	}

	cranex.x = minx;
	cranex.y = miny;

	loadvector = mul2d( (-0.5 / norm2d(cranex)), cranex);
	loadvector = sum2d(v2d(0.0,-1.0),loadvector);
	loadvector = mul2d( 1.2 * TARGET_LOAD / LOAD_STEPS, loadvector);
#ifdef PRINTS
printf("cr=(%f %f)lv=(%f %f)",cranex.x,cranex.y,loadvector.x,loadvector.y);
#endif
/* test crane resistance */
for (k=0;(k<LOAD_STEPS) && 
(.9999 <= (addforce(&bodies MINE[where], cranex,loadvector)))
; k++)  ;
;
#ifdef PRINTS
printf("[k=%d]",k);
#endif
ret = (REAL) k / LOAD_STEPS;

ret = 1.0 + ((-minx) * ret);

#ifdef PRINTS
printf("FIT %f ",ret);
#endif
return ret;
}


#define cmemcpy(p,q,a,b) memcpy(p,q,(a)*(b))


static char bricksizes[] = {4,6,8,10,12,16};
#define BRICKSIZES 6
#define MAXJOINTSIZE 6

#ifndef GLIB
#define gbrick brick
#define gbrickcount brickcount
#define gbricks bricks
#define glibid size
#endif


typedef struct {
	gbrick *gbricks;
	int brickcount;
	REAL fitness;
	REAL mass;
#ifdef PARALLEL
	unsigned long lock;
#endif
} individual;


individual population[POP_SIZE];

int popcount = 0;

#ifdef PARALLEL
lock_ind(int i, int lock) {
	test_then_add(&(population[i].lock),lock);
}
#endif

void update_population();

void start_pop() {
	USEMINE
	int j;
	popcount = 0;
	bricks MINE[0].size = 6;
	for (j=0;j<4;j++) {
		bricks MINE [0].joints[j]= EMPTY;
		bricks MINE [0].jointsize[j]=0;
	}
	brickcount MINE=1;
	if (makemodel() || tooheavy()) {
		fprintf(stderr,"Error in start pop 1\n");
		exit(1);
	}

	fitnessvalue MINE = fitness();
	update_population();
}


int bricksize(int x) {
	if (x<0) return bricksizes[0];
	if (x>=BRICKSIZES) return bricksizes[BRICKSIZES-1];
	return bricksizes[x];
}

int randombricksize() {
	return bricksize(RANDINT() % BRICKSIZES);
}

/**
 * Return mutated size of brick for a brick of size s.
 * Return either the next bigger or next smaller valid brick size after s. 
 * It may return s if there are no bigger (resp. smaller) valid sizes. 
 */

int mutatebricksize(int s) {
	int x;
	for (x=0;x<BRICKSIZES && s != bricksizes[x];x++);
	if (x>=BRICKSIZES) {
		fprintf(stderr,"mutatebricksize: invalid size %d\n",s); exit(1);
	}
	return bricksize(x + (RANDINT()%2)*2 -1);
}

int randomjointsize() {
	return (RANDINT() % MAXJOINTSIZE) + 1;
}

/** Return mutated joint size for a joint of size s.
 * 
 * Return s+-1 but no more than 6 no less than 1. 
 */
int mutatejointsize(int s) {
	s = s + (RANDINT()%2)*2 +1;
	if (s>6) return 6;
	else if (s<1) return 1;
	return s;
}

int mutate(brick *somebricks, int acount) {
	USEMINE
	int i,j,k,l;
	brick abrick;
	LOCAL brick atree[MAXBRICKS];

	cmemcpy(atree,somebricks,sizeof *atree,acount);
	initgrid();
	brickcount MINE=0;
#ifdef PRINTS
	printf("MUTATE s=%d",acount);
#endif
	// Choose one of the bricks, or the root
	i= (RANDINT()  % (acount + 1)) -1;
	if (i==-1) {   
		// Root. mutate size of root brick
#ifdef PRINTS
		printf("size mut %d",atree[0].size);
#endif
		atree[0].size = mutatebricksize(atree[0].size);
#ifdef PRINTS
		printf("->%d ",atree[0].size);
#endif
		if (loadtree(0,0,atree,UNUSED,0,NULL,0)) return 1;
	}
	else {
		// Brick. Choose a random joint j for brick i 
		j = RANDINT() % 4;
#ifdef PRINTS
		printf("br %d j %d (old %d %d)",i,j,atree[i].joints[j],
				atree[i].jointsize[j]);
#endif
		if ((k=atree[i].joints[j])==EMPTY) {
#ifdef PRINTS
			printf("NEW %d",j);
#endif
			if (acount+1 == (MAXBRICKS-PREDEF_BRICK_NO)) return 1; /* failed: can't add more */
			atree[i].joints[j]=TRANSIT;
			for (l=0;l<4;l++) {
				abrick.joints[l]=EMPTY;
				abrick.jointsize[l]=0;
			}    
			atree[i].jointsize[j]=randomjointsize();
			abrick.size=randombricksize();
#ifdef PRINTS
			printf("/%d/%d/",abrick.size,atree[i].jointsize[j]);
#endif
			if (loadtree(0,0,atree,TRANSIT,0,&abrick,0)) 
				return 1;
		}
		else {
			atree[k].size=mutatebricksize(atree[k].size);
			atree[i].jointsize[j]=mutatejointsize(atree[i].jointsize[j]);
#ifdef PRINTS
			printf("/%d/%d/",atree[k].size,atree[i].jointsize[j]);
#endif
			if (loadtree(0,0,atree,UNUSED,0,NULL,0)) return 1;
		}    
#ifdef PRINTS
		printf("now %d.%d=%d(%d)js %d ",i,j,bricks MINE[i].joints[j],
				bricks MINE[k].size,bricks MINE[i].jointsize[j]);
#endif
	}
	if (makemodel()) {
		fprintf(stderr,"Error: Makemodel failed at mutate\n");
		/* exit(1); */ return 1; /* mutation failed */
	}
	if (tooheavy()) return 1;   /* collapsed! */
	fitnessvalue MINE= fitness();
	return 0;                /* mutation successful !*/
}

/** 
 * Load structure rooted at a into the bricks data structure, marking the grid as
 * we proceed.
 * 
 * First, place brick *a at (gx,gy) (if impossible, return 1)
 * Then place a's descendants, as possible (by calling loadtree recursively) 
 * and return 0. 
 * 
 * Aoff is the array offset of *a with respect to the original encoding (e.g. *a = x[aoff])  so 
 * that when a joint calls upon brick k, the final offset is a[i-aoff] (= x[i]).
 *  
 * If at some point a joint labelled acut is found, insert brick b (offset boff) there 
 * and its descendants. 
 * 
 * Remember to clear bricks & grid before calling. 
 * 
 * If not inserting anything, set acut to the constant UNUSED, b to NULL and boff to 0. 
 * 
 * TODO: If I were to rewrite this function I'd probably replace the confusing sense of aoff and use
 * a straightforward offset instead (i.e. loadtree(gx,gy,&a[aoff],acut,aoff,...) would be replaced by 
 * loadtree(gx,gy,a,acut,aoff,...). 
 */
int loadtree(int gx,int gy,brick *a, int acut, int aoff, brick *b, int boff) {
	USEMINE
	int j,k,joint,i;
	if (brickcount MINE == (MAXBRICKS-PREDEF_BRICK_NO)) return 1; /* too big; failed */
	if (putbrickat(gx,gy,a[0].size)) 
		return 1; /* doesn't fit */
	k=brickcount MINE++;
	bricks MINE [k].size = a[0].size;
	for (j=0;j<4;j++) {
		joint = a[0].joints[j];
		if (!ISBRICK(joint)) {
			bricks MINE [k].joints[j] = joint;
			bricks MINE [k].jointsize[j] = a[0].jointsize[j];
		}
		else /* if (joint >= 0) */ {
			bricks MINE [k].joints[j] = brickcount MINE ;
			bricks MINE [k].jointsize[j] = a[0].jointsize[j];
			if (joint == acut) 
				i = loadtree(gx+XSHIFT(j,a[0].jointsize[j],
						a[0].size,b[0].size),
						gy+YSHIFT(j),
						b,UNUSED,boff,NULL,0);
			else
				i = loadtree(gx+XSHIFT(j,a[0].jointsize[j],
						a[0].size,a[joint-aoff].size),
						gy+YSHIFT(j),
						&a[joint-aoff],acut,joint,b,boff);
			if (i) {
				/* didn't fit, cut off rest */
				bricks MINE[k].joints[j]=EMPTY;
				bricks MINE[k].jointsize[j]=0;
			}
		}
	}
	return 0;
}


int crossover(gbrick a[], int acount, gbrick b[], int bcount) {
	USEMINE
	int i,j,x;

	i = RANDINT() % acount;
	j = RANDINT() % bcount;
	brickcount MINE=0;
#ifdef GLIB
	gbrickcount=0;
#endif
#ifdef PRINTS
	printf("CROSS (%d)%d (%d)%d>",i,acount,j,bcount);
#endif
	initgrid();
	if (i==0) 
		x=loadtree(0,0,&b[j],UNUSED,j,NULL,0);
	else 
		x=loadtree(0,0,&a[0],i,0,&b[j],j);
	if (x) return 1;            /* didn't fit */
	if (makemodel()) return 1;  /* failed */
	if (tooheavy()) return 1;   /* collapsed! */
	fitnessvalue MINE = fitness();
	return 0;                /* crossover successful !*/
}

int select_on_fitness(int inverse /*REAL alpha*/ LOCK_PARAM) {
	REAL x,end,acc;
	int i;

#ifdef PARALLEL
	int done;
#endif

#ifdef PARALLEL
	done=0;
	while (!done) {
#endif
		end = acc = 0.0;
		for (i=0;i<popcount;i++) 
			/*    end += pow(population[i].fitness,alpha); */
			end += inverse ? (1.0 / population[i].fitness) : population[i].fitness;
			x = (REAL) (RANDINT() % 1000000) / 1000000.0 * end;
			for (i=0;i<popcount;i++) {
				/* acc += pow(population[i].fitness,alpha); */
				acc += inverse ? (1.0 / population[i].fitness) : population[i].fitness;
				if (acc+NOTHING > x) break;
			}
#ifdef PARALLEL
			if (i<popcount) {
				LOCK
				if (to_write) {
					if (population[i].lock == 0) {
						population[i].lock = WRITE_LOCK;
						done=1;
					}
				}
				else {
					if ((population[i].lock & WRITE_LOCK)==0) {
						population[i].lock++;
						done=1;
					}
				}
				UNLOCK
			}
	}
#else
#ifdef CHECKS
	if (i<0 || i>= popcount)   
		fprintf(stderr,"out of bounds in select_on_fitness\n");
#endif
#endif
	return i;  
}



#define MMS(x,ma,mi,su) su += x; if(x > ma) ma = x; if(x < mi) mi = x;

void watch(count) {
	USEMINE
	int i,best,worst;
	int minbricks,maxbricks,sumbricks;
	REAL minfit,maxfit,sumfit;
	REAL minmass,maxmass,summass;
	static int elapsed = 0;
	static int lastringcount = 1;
	static FILE *fp = NULL;

	WHOAM2
	fprintf(stderr,"watch %d\n",count);
	if (fp==NULL) fp=fopen("watch","w");
	minbricks = 100;
	maxbricks = -100;
	sumbricks = 0;
	minfit = minmass = 1e32;
	maxfit = maxmass = -1e32;
	sumfit = summass = 0.0;
	fprintf (fp,"%6d %d ",count,popcount);
	for (i=0;i<popcount;i++) {
#ifdef PARALLEL
		if ((population[i].lock & WRITE_LOCK)==0) {
#endif
			MMS(population[i].brickcount,maxbricks,minbricks,sumbricks);
			MMS(population[i].fitness,maxfit,minfit,sumfit);
			MMS(population[i].mass,maxmass,minmass,summass);
			if (population[i].fitness == maxfit) best=i;
			if (population[i].fitness == minfit) worst=i;
#ifdef PARALLEL
		}
#endif
	}
	fprintf (fp,"fi %f %f %f br %d %d %f ma %f %f %f ",
			maxfit,minfit,sumfit / popcount,
			maxbricks,minbricks, (REAL) sumbricks / popcount,
			maxmass,minmass,summass / popcount);
	initgrid();
	brickcount MINE=0;
#ifdef GLIB
	gbrickcount=0;
#endif
	if(loadtree(0,0,population[best].gbricks,UNUSED,0,NULL,0)
			|| makemodel() || tooheavy()) {
		fprintf(stderr,"Error: can't load best\n");
		dumpfile("err",population[best].gbricks, population[best].gbrickcount, 
				bodies MINE, bodycount MINE, 
				joints MINE, jointcount MINE);
		exit(1);
	}
	mbricks("BEST.m",bricks MINE,brickcount MINE,fitness(),
			bodies MINE,bodycount MINE,joints MINE,jointcount MINE);
	initgrid();
	brickcount MINE=0;
	gbrickcount MINE=0;
	loadtree(0,0,population[worst].gbricks,UNUSED,0,NULL,0);
	makemodel();
	tooheavy();
	mbricks("WORST.m",bricks MINE,brickcount MINE,fitness(),
			bodies MINE,bodycount MINE,joints MINE,jointcount MINE);

	i=time(NULL);
	if (elapsed) fprintf(fp,"sec %d rc %lu ",i-elapsed,ringcount-lastringcount);
	if (elapsed) fprintf(stderr,"=%d secs.",i-elapsed);
	lastringcount=ringcount;
	elapsed=i;
	fprintf(fp,"\n");
	fflush(fp);
#ifdef GLIB
	gwatch(count);
#endif
}



REAL maxfitness,minfitness;


void save_ind_at(int i) {
	USEMINE

#if (defined PARALLEL) && (defined CHECKS)
	if (population[i].lock != WRITE_LOCK) 
		fprintf(stderr,"warning: save at %d lock %x\n",i,population[i].lock);
#endif
	population[i].gbricks = (gbrick *) calloc(gbrickcount MINE, 
			sizeof *(gbricks MINE));
	if (population[i].gbricks == NULL) {
		perror("OUT OF MEM save_ind_at");
		exit(1);
	}
	cmemcpy(population[i].gbricks, gbricks MINE, gbrickcount MINE, sizeof *(gbricks MINE));
	population[i].brickcount = gbrickcount MINE;
	population[i].fitness = fitnessvalue MINE;
	population[i].mass = totalmass MINE;
#ifdef PRINTS
	printf("SA %d bc=%d f=%f m=%f\n",
			i,gbrickcount MINE,fitnessvalue MINE, totalmass MINE);

	for (i=0;i<gbrickcount MINE;i++) {
		printf("gbrick %d size %d joints ",
				i,
				gbricks MINE[i].glibid);
#ifdef GLIB
		for (j=0;j<glib[gbricks[i].glibid].jointcount;j++) {
#else 
			for (j=0; j<4; j++) {
#endif
				printf("%d(%d) ",gbricks MINE [i].joints[j],gbricks MINE [i].jointsize[j]);
			}
			printf("\n");
		}
		printf("-----------\n");
#ifdef GLIB
		if (gverify(gbricks,gbrickcount)) {
			fprintf(stderr,"verify failed\n");
			exit(1);
		}
#endif
#endif
	}

	int parent ONEFOREACH;

	void update_population() {
		USEMINE
		int i,j;
		char s[50],t[50];
		static unsigned long count=0;
		unsigned long mycount;
#ifdef PARALLEL
		int locked;
#endif 
		mycount = INCREMENT(count);

#ifdef MAKELINEAGE
		if (lineage==NULL) lineage=fopen("lineage","w");
#endif
#ifdef SAVELINEAGE
		if (lineage==NULL) {
			lineage=fopen("linlist","r");
			fscanf(lineage,"%d %d %d",&nextsave,&i,&j);
			printf("next save at %d\n",nextsave);
		}
#endif

#ifdef GLIB
		for (k=0;k<gbrickcount;k++)
			glib[gbricks[k].glibid].usecount++;
#endif
#ifdef PARALLEL
		if (popcount < POP_SIZE) {
			LOCK
			locked=1;
		}
		else locked=0;
#endif

		if (popcount == POP_SIZE) {
#ifdef PARALLEL
			if (locked) UNLOCK
#endif
			i=select_on_fitness(/*-ALPHA*/1 TO_WRITE);
#ifdef GLIB
			remove_pop(&population[i]); 
#else
			free(population[i].bricks);
#endif
		}
		else {
			i=popcount++;
#ifdef PARALLEL
			population[i].lock = WRITE_LOCK;
			population[i].fitness = 1.0;
			UNLOCK
#endif
		}

#ifdef MAKELINEAGE
		fprintf(lineage,"%d %d %d\n",count,parent,i);
		fflush(lineage);
#endif
		save_ind_at(i);
#ifdef PARALLEL
		test_then_add(&(population[i].lock),-WRITE_LOCK);
#endif
#ifdef SAVELINEAGE
		if (mycount >= nextsave) {
			printf("saving %d %d\n",mycount,nextsave);
			sprintf(t,"%5d",savecount++);
			for (i=0;t[i]!='\0';i++) if (t[i]==' ' ) t[i]='0';
			sprintf(s,"LIN%s.m",t);
			mbricks(s,bricks MINE,brickcount MINE,fitnessvalue MINE,
					bodies MINE, bodycount MINE, joints MINE, jointcount MINE);

			fscanf(lineage,"%d %d %d",&nextsave,&j,&k);
			printf("next save at %d\n",nextsave);

		}
#endif

		count++;

		if (fitnessvalue MINE > maxfitness) {
			LOCK
			if (fitnessvalue MINE > maxfitness) {
				maxfitness=fitnessvalue MINE;
				sprintf(t,"%6d",(int) (fitnessvalue MINE*1000));
				for (j=0;t[j]!=0;j++) if (t[j]==' ') t[j]='0';
				WHOAM2
				fprintf(stderr,"MAX %s\n",t);
				sprintf(s,"MAX%s.m",t);
				mbricks(s,bricks MINE,brickcount MINE,fitnessvalue MINE,
						bodies MINE, bodycount MINE, joints MINE, jointcount MINE);
				unlink("MAX.m");
				link(s,"MAX.m");
				sprintf(s,"MAX%s.info",t);
				dumpfile(s,bricks MINE, brickcount MINE, bodies MINE, bodycount MINE, 
						joints MINE, jointcount MINE);
				unlink("MAX.info");
				link(s,"MAX.info");
#ifdef VBLOCKS
				sprintf(s,"MAX%s.vb",t);
				vb_bricks(s,bricks,brickcount,fitnessvalue);
				unlink("MAX.vb");
				link(s,"MAX.vb");
#endif
#ifdef GLIB
				sprintf(s,"MAX%s.glib",t);
				glibsave(s);
				unlink("MAX.glib"); 
				link(s,"MAX.glib");
#endif
				UNLOCK
			}
		}
	}

	unsigned long compt =0, compf = 0, mutat = 0, mutaf = 0, crost = 0, crosf=0;

	void evolve() {
		USEMINE
		int i,j;
#ifdef PARALLEL
		static unsigned long bigcount=0;
		static int rseed;
#endif
		int status;
		int count;


#ifdef PARALLEL
		count=INCREMENT(bigcount);
#ifdef PRINTS
		if(myid==0) rseed=SRANDOM;
		m_sync();
		printf("(%d) init evolve count=%d\n",myid,count);
		i=myid+rseed;
		RANDSEED(i);
		printf("(%d) init randseed=%d\n",myid,i);
#endif
#else
		count=0;
#endif

		while (maxfitness < TARGET_FITNESS && count < COUNTLIMIT) {
#if defined(PRINTS) && defined(PARALLEL)
			printf("(%d)evolve loop max=%f count=%d\n",myid,maxfitness,count);
#endif
			if ((!NOCROSSOVER) && RANDINT() % 2) {
				/* crossover */
				i=select_on_fitness(0 /*ALPHA*/ TO_READ);
				j=select_on_fitness(0 /*ALPHA*/ TO_READ);
#ifdef PRINTS
				printf("co %d %d ",i,j);
#endif
				parent MINE=i;
				INCREMENT(crost);
				status=crossover(
						population[i].gbricks,
						population[i].brickcount-PREDEF_BRICK_NO,
						population[j].gbricks,
						population[j].brickcount-PREDEF_BRICK_NO);
#ifdef PARALLEL
				test_then_add(&(population[i].lock),-1);
				test_then_add(&(population[j].lock),-1);
#endif
				if (status==0) update_population();
				else {
					INCREMENT(crosf);
#ifdef PRINTS
					printf("co fail\n");
#endif
				}
			}
			else {
				/* mutate */
#ifdef GLIB
				if (NOCOMPRESS || RANDINT() % 4)
#else
					if (1)
#endif
					{
						/* plain mutation */
						i=select_on_fitness(0 /*ALPHA*/ TO_READ);
#ifdef PRINTS
						printf("mut %d ",i);
#endif
						parent MINE=i;
						INCREMENT(mutat);
						status=mutate(population[i].gbricks,
								population[i].brickcount-PREDEF_BRICK_NO);

#ifdef PARALLEL
						test_then_add(&(population[i].lock),-1);
#endif
						if (status==0)
							update_population();
						else {
							INCREMENT(mutaf);
#ifdef PRINTS
							printf("mut fail\n");
#endif
						}
					}
#ifdef GLIB
					else {
						/* compression */
						i=select_on_fitness(0 /*ALPHA*/);
#ifdef PRINTS
						printf("comp %d ",i);
#endif
						parent MINE=i;
						compt++;
						if (0==compress(population[i].gbricks,population[i].brickcount))
							update_population();
						else {
							compf++;
#ifdef PRINTS
							printf("comp fail\n");
#endif
						}
					}
#endif
			}
#ifdef PARALLEL
			count=INCREMENT(bigcount);

			if ((count % WATCH_COUNT) == 0) { LOCK watch(count); UNLOCK }; 
			/*

    if ((i=m_next()) >= WATCH_COUNT-PROCESSORS) {
#ifdef CHECK
      if (haslocked[myid]) {
	fprintf(stderr,"panic: goint into sync with lock\n");
      }
#endif
      m_sync();
      if (myid == 0) watch(count);
      m_sync();		
    }
			 */
#else
			count++;
			if ((count % WATCH_COUNT)==0) watch(count);
#endif
		}
	}

	mtree(FILE *fp,int gx, int gy, int brickno, brick bricks[]) {
		brick *b;
		int j,k;

		b=&bricks[brickno];
		fprintf(fp,"plot([%f %f %f %f %f],[%f %f %f %f %f],'c');\n",
				(REAL)gx, (REAL)gx+b->size, (REAL)gx+b->size,
				(REAL)gx, (REAL)gx,
				(REAL)gy*HEIGHT,(REAL)gy*HEIGHT,
				(REAL)(gy+1)*HEIGHT,(REAL)(gy+1)*HEIGHT,
				(REAL)gy*HEIGHT);
		fprintf(fp,"text(%f,%f,'%d(%d)');\n",
				(REAL) gx, (REAL)gy*HEIGHT + 0.5, brickno,bricks[brickno].size);

		for (j=0;j<4;j++) {
			k=b->joints[j];    
			if (k != EMPTY && k != GROUND) {
				mtree(fp,
						(gx + XSHIFT(j,bricks[brickno].jointsize[j],
								bricks[brickno].size,bricks[k].size)),
								gy + YSHIFT(j),k,bricks);
				fprintf(fp,"text(%f,%f,'%d');\n",
						(REAL) gx + ((j==0 || j==3) ? 0 : bricks[brickno].size - 2),
						(REAL) HEIGHT * (gy + ((j==0 || j==1) ? 0 : 1)),
						bricks[brickno].jointsize[j]);
			}

		}
	}

	mbricks(char *name,brick bricks[],int brickno,double fit,body bodies[], int bodycount,
			joint joints[], int jointcount) {
		FILE *fp;
		body *b;
		int i,j;
		int gx,gy;
		char *code;

		brickno++;

		fp=fopen(name,"w");
		fprintf(fp,"figure; hold on\n");
		mtree(fp,0,0,0,bricks);
		for (i=0;i<bodycount;i++) {
			b = &bodies [i];
			fprintf(fp,"plot(%f,%f,'co');\n",b->x.x,b->x.y);
			for (j=0;j < b->jointcount;j++) {
				fprintf(fp,"plot(%f, %f,'y*');\n",
						b->joints[j]->x.x,
						b->joints[j]->x.y);    
				fprintf(fp,"x=plot([%f %f],[%f %f]);\n",
						b->x.x,b->joints[j]->x.x,
						b->x.y,b->joints[j]->x.y,code);
				fprintf(fp,"set(x,'Color',[1 %f 0]);\n",
						fabs(b->joints[j]->torques/b->joints[j]->maxtorque));
				fprintf(fp,"set(x,'LineWidth',%d);\n",
						(int) (7 * b->joints[j]->maxtorque / 28000) + 1);
			}
		}
		fprintf(fp,"set(gca,'Title',text(0,0,'%f'))\n",fit);
		fclose(fp);
	}

#ifdef VBLOCKS
	vb_tree(FILE *fp,int gx, int gy, int brickno) {
		brick *b;
		int j,k;

		b=&bricks[brickno];
		fprintf(fp,"%d     %d %d    %d %d %d       %d %d %d\n",
				1,
				1,b->size,
				0,(gy-4)*3,-(gx+b->size),
				0,0,0);
		for (j=0;j<4;j++) {
			k=b->joints[j];    
			if (k != EMPTY && k != GROUND) {
				vb_tree(fp,
						(gx + XSHIFT(j,bricks[brickno].jointsize[j],
								bricks[brickno].size,bricks[k].size)),
								gy + YSHIFT(j),k);
			}
		}
	}

	vb_bricks(char *name,brick bricks[],int brickno,double fit) {
		FILE *fp;
		body *b;
		int i,j;
		int gx,gy;
		char *code;

		brickno++;

		fp=fopen(name,"w");
		fprintf(fp,"2 	10 40 	-5 -13 -40	 0 	7 	0\n");
		vb_tree(fp,0,0,0);
		fclose(fp);
	}


#endif

	dumpfile(char *name, brick bricks[], int brickcount, body bodies[], int bodycount,
			joint joints[], int jointcount) {
		FILE *fp;
		int i,j;

		fp=fopen(name,"w");
		fprintf(fp,"%d\n\n",brickcount);
		for (i=0;i<brickcount;i++) {
			fprintf(fp,"%5d %d ",i,bricks[i].size);
			for (j=0;j<4;j++) {
				fprintf(fp,"%d %d   ",bricks[i].joints[j],bricks[i].jointsize[j]);
			}
			fprintf(fp,"\n");
		}

		fprintf(fp,"\n\n---------bodies(%d)-------\n\n",bodycount);
		for (i=0;i<bodycount ;i++) {
			fprintf(fp,"%5d=%8x (%f,%f) eq %d mass %f ",i,&bodies[i],
					bodies[i].x.x,
					bodies[i].x.y,bodies[i].equilibrium,
					bodies[i].mass);
			for (j=0;j<bodies[i].jointcount;j++) 
				fprintf(fp,"%x(%x %d) ",bodies[i].joins[j],
						bodies[i].joints[j],bodies[i].jointsign[j]);
			fprintf(fp,"\n");
		}

		fprintf(fp,"\n\n-----------joints(%d)----------\n\n",jointcount);
		for (i=0;i<jointcount;i++) {
			fprintf(fp,"%5d=%8x (%f,%f) t=%f f=%f,%f, mt=%f\n",
					i,
					&joints[i],
					joints[i].x.x,joints[i].x.y,
					joints[i].torques,joints[i].forces.x,joints[i].forces.y,
					joints[i].maxtorque);
		}

		fclose(fp);
	}


	main(int argc,char **argv) {
		int i,j,x,y,z;
		FILE *in;
		int myid=0;


#ifdef PARALLEL
		for (i=0;i<PROCESSORS;i++) {
			haslocked[i]=0;
		}
#endif
		if (argc==1) {
			start_pop();
			minfitness=1e32;
			maxfitness=-1e32;
		}
		else {
			printf("loading %s...\n",argv[1]);
			in=fopen(argv[1],"r");
			fscanf(in,"%d",&brickcount);
			for (i=0;i<brickcount MINE;i++) {
				fscanf(in,"%d %d",&z,&x);
				printf("%d %d ",z,x);
				bricks MINE[i].size = x;
				for (j=0;j<4;j++) {
					fscanf(in,"%d %d",&x,&y);
					bricks MINE [i].joints[j] = x;
					bricks MINE [i].jointsize[j] = y;
					printf("%d(%d) ",x,y);
				}
				printf("\n");
			}
			if (makemodel()) {
				fprintf(stderr,"first makemodel failed!\n");
				mbricks("fail.m",bricks MINE,brickcount MINE,0.0, bodies MINE, bodycount MINE, joints MINE,
						jointcount MINE);
				KILL
				exit(1);
			}
			if (tooheavy()) {
				fprintf(stderr,"addforcecount = %d\n",addforcecount MINE );
				fprintf(stderr,"first tooheavy failed! dumping fail.m\n");
				mbricks("fail.m",bricks MINE,brickcount MINE,0.0, bodies MINE, bodycount MINE, joints MINE,
						jointcount MINE);
				KILL
				exit(1);
			}
			fitnessvalue MINE = minfitness = fitness();
			maxfitness = -1e32;
			mbricks("START.m",bricks MINE,brickcount MINE,
					fitnessvalue MINE,bodies MINE, bodycount MINE,
					joints MINE, jointcount MINE);
			dumpfile("START.info",bricks MINE,brickcount MINE,
					bodies MINE,bodycount MINE,joints MINE,jointcount MINE);
			popcount=0;
			update_population();
		}
		i=SRANDOM;
		fprintf(stderr,"fit (%s) starts with seed %d\n",argv[0],i);
		RANDSEED(i);
#ifdef PARALLEL
		m_set_procs(PROCESSORS);
		hasforked=1;
		m_fork(evolve);
#else
		evolve();
#endif
	}

	xymain() {
		int x,y,z;
		initgrid();
		while (1) {
			scanf("%d %d",&x,&y);
			printf("%d %d = %d\n",x,y,getgrid(x,y));
			printf("new value-> ");
			scanf("%d",&z);
			printf("%d %d (%d) = %d == ",x,y,setgrid(x,y,z),z);
			printf("%d \n",getgrid(x,y));
		}
	}


