/*---------------------------------------------------------------
This file is part of the Buildable Evolution Software Project. 

http://www.demo.cs.brandeis.edu/pr/buildable
(C) Pablo Funes and Jordan B. Pollack 1997-2008
pablo@cs.brandeis.edu

The methods and configurations incorporated in the Builable Evolution
Software Project are covered in part by United States patent
6,434,492 "Computer apparatus and method for analyzing structural
stability" of August 13, 2002. 

The Buildable Evolution Software Project is free software: you 
can redistribute it and/or modify it under the terms of the 
GNU General Public License as published by the Free Software 
Foundation, either version 3 of the License, or (at your option) 
any later version. 

The Buildable Evolution Software Project is distributed in the
hope that it will be useful, but WITHOUT ANY WARRANTY; without
even the implied warranty of MERCHANTABILITY or FITNESS FOR A
PARTICULAR PURPOSE. See the GNU General Public License for more
details. 

You should have received a copy of the GNU General Public License 
along with the software. If not, see <http://www.gnu.org/licenses/>. 
---------------------------------------------------------------*/
/* reals and booleans */

#define REAL float

#define BOOL int
#define TRUE 1
#define FALSE 0

#define NOTHING 0.000001
#define MIN(a,b) ((a) > (b)) ? (b) : (a)
#define MAX(a,b) ((a) < (b)) ? (b) : (a)
#define SIGN(a) (((a) == 0) ? 0 : (((a) > 0) ? 1 : -1))

/* 2d vectors and their operations */

typedef struct {
  REAL x,y;
} vector2d;


vector2d v2d(REAL x, REAL y);

REAL reallyzero(REAL x) ;

REAL norm2d(vector2d v) ;

REAL dot2d(vector2d v, vector2d w) ; 

vector2d sum2d(vector2d v, vector2d w);


vector2d mul2d(REAL a, vector2d v); 

vector2d sub2d(vector2d v, vector2d w);

REAL dist2d(vector2d v, vector2d w);

REAL det2d(vector2d v, vector2d w) ;

BOOL zero2d(vector2d v) ;
BOOL reallyzero2d(vector2d v) ;

BOOL parallel2d(vector2d v, vector2d w) ;

/* forces and torques */
typedef vector2d force2d;

typedef struct {
  force2d f;
  vector2d x;
} relforce;

typedef REAL torque2d;

torque2d torqueat(vector2d mass, vector2d x, force2d f) ;

/* bodies and joints */

#define BODYJOINTS 10

typedef struct {
  vector2d x;
  torque2d torques;
  force2d forces;
  torque2d maxtorque;
  int n;
} joint;

typedef struct body_struct{
  vector2d x;
  BOOL equilibrium;
  joint *joints[BODYJOINTS];
  struct body_struct  *joins[BODYJOINTS];
  int jointsign[BODYJOINTS];
  int jointcount;
  REAL mass;
  int n;
} body ;


REAL addforce(body *b, vector2d x, force2d f) ;


void addjoint(body *b1, int sign, body *b2, joint *j) ;














